﻿using AIWolf.Lib;
using System.Collections.Generic;
using System.Linq;

namespace WhiteWolf.Arrangement
{
    public class DayGameInfo : IDayGameInfo
    {

        IGameInfo dayTimeLatestGameInfo;

        IGameInfo nightLatestGameInfo;

        IGameInfo tomorrowGameInfo;

        /// <summary>会話のstring→Content変換器。初期値はプロトコル用。</summary>
        public ITalkEncoder TalkCoder { get; set; } = new ProtocolEncoder();

        // getter、setterを定義しているものは、役職により設定されるタイミングが異なる。
        // (夜にupdateが呼ばれないので翌日昼に初めて受け取る)

        IGameInfo LatestGameInfo => nightLatestGameInfo ?? dayTimeLatestGameInfo;

        public int Day => LatestGameInfo.Day;

        public IEnumerable<Agent> DayTimeAliveAgent => dayTimeLatestGameInfo.AliveAgentList;


        public List<ContentUtterance> talkList = new List<ContentUtterance>();

        public IEnumerable<ContentUtterance> TalkList => talkList;

        List<VoteCollection> voteList = new List<VoteCollection>();
        public IReadOnlyList<VoteCollection> VoteList => voteList;

        public Agent ExecuteAgent { get; private set; }

        public IEnumerable<Agent> NightAliveAgent { get; private set; }

        public IEnumerable<Agent> AttackDeadAgent => tomorrowGameInfo?.LastDeadAgentList;

        public Agent TryAttackAgent => tomorrowGameInfo?.AttackedAgent;

        public Agent GuardAgent => tomorrowGameInfo?.GuardedAgent;

        List<VoteCollection> attackVoteList = new List<VoteCollection>();
        public IReadOnlyList<VoteCollection> AttackVoteList => attackVoteList;

        public List<ContentUtterance> whisperList = new List<ContentUtterance>();
        public IEnumerable<ContentUtterance> WhisperList => whisperList;

        public AIWolf.Lib.Judge DivineResult => tomorrowGameInfo?.DivineResult;

        public AIWolf.Lib.Judge MediumResult => tomorrowGameInfo?.MediumResult;


        public void Update(IGameInfo gameInfo)
        {
            if (gameInfo.Day == Day)
            {
                if (gameInfo.LatestExecutedAgent == null)
                {
                    dayTimeLatestGameInfo = gameInfo;
                }
                else
                {
                    // 夜になった＆夜の行動がある役職
                    nightLatestGameInfo = gameInfo;
                    ExecuteAgent = gameInfo.LatestExecutedAgent;
                    NightAliveAgent = gameInfo.AliveAgentList;
                }

                UpdateTalk(gameInfo);
                UpdateVote();
                return;
            }

            if (gameInfo.Day == Day + 1)
            {
                UpdateFromTomorrowGameInfo(gameInfo);
                return;
            }
        }

        private void UpdateFromTomorrowGameInfo(IGameInfo gameInfo)
        {
            tomorrowGameInfo = gameInfo;

            if (NightAliveAgent == null)
            {
                // 夜のデータが設定されていない場合(夜の行動が無い役職)
                ExecuteAgent = gameInfo.ExecutedAgent;
                NightAliveAgent = new List<Agent>(DayTimeAliveAgent.Where(a => !a.Equals(gameInfo.LatestExecutedAgent)));

                voteList.Add(new VoteCollection(gameInfo.VoteList));
            }
            else
            {
                // 夜のデータが設定されている場合(夜の行動が有る役職)
                if (gameInfo.AttackVoteList.Count > 0)
                {
                    attackVoteList.Add(new VoteCollection(gameInfo.AttackVoteList));
                }
            }
        }

        public DayGameInfo(IGameInfo gameInfo)
        {
            dayTimeLatestGameInfo = gameInfo;
            if (Day == 0)
            {
                nightLatestGameInfo = gameInfo;
            }
        }

        private void UpdateVote()
        {
            // 追放再投票になった直後の処理
            // 現ルールは再投票１回なので大丈夫だが、再投票２回以上のルールの場合は要改良
            if (dayTimeLatestGameInfo.LatestExecutedAgent == null && dayTimeLatestGameInfo.LatestVoteList.Count > 0)
            {
                voteList.Add(new VoteCollection(dayTimeLatestGameInfo.LatestVoteList));
            }

            // 襲撃再投票になった直後の処理
            // 現ルールは再投票１回なので大丈夫だが、再投票２回以上のルールの場合は要改良
            if (dayTimeLatestGameInfo.LatestAttackVoteList.Count > 0)
            {
                attackVoteList.Add(new VoteCollection(dayTimeLatestGameInfo.LatestAttackVoteList));
            }

            // 夜になった直後の処理
            if (dayTimeLatestGameInfo.LatestExecutedAgent != null)
            {
                voteList.Add(new VoteCollection(nightLatestGameInfo.VoteList));
            }
        }

        private void UpdateTalk(IGameInfo gameInfo)
        {
            var addTalk = gameInfo.TalkList.Where((talk, count) => count >= talkList.Count);
            addTalk.ToList().ForEach(talk => talkList.Add(new ContentUtterance(talk, TalkCoder)));
        }

    }
}
