﻿using AIWolf.Lib;
using System.Collections.Generic;

namespace WhiteWolf.Arrangement
{
    /// <summary>
    /// 全日分のゲーム情報
    /// </summary>
    public interface ITotalityGameInfo
    {
        /// <summary>ゲーム設定</summary>
        IGameSetting GameSetting { get; }

        /// <summary>各日の情報 Key=日</summary>
        SortedDictionary<int, IDayGameInfo> DayGameInfo { get; }

        /// <summary>最新日の情報</summary>
        IDayGameInfo TodayGameInfo { get; }

        /// <summary>前日の情報</summary>
        IDayGameInfo YesterdayGameInfo { get; }

        /// <summary>自分のエージェント</summary>
        Agent Me { get; }

        /// <summary>自分の役職</summary>
        Role MyRole { get; }

        /// <summary>エージェント</summary>
        IReadOnlyList<Agent> AgentList { get; }

        /// <summary>最新の生存エージェント</summary>
        IReadOnlyList<Agent> AliveAgentList { get; }


        /// <summary>人狼のエージェント（自分が人狼でない場合は空リスト）</summary>
        IReadOnlyList<Agent> WerewolfAgentList { get; }


        /// <summary>全視点のシステム情報のみでフィルタした視点情報</summary>
        IViewpoint AllViewSystemInfo { get; }

        /// <summary>全視点のCO・判定情報を含めてフィルタした視点情報</summary>
        IViewpoint AllViewTrustInfo { get; }

        /// <summary>自分視点のシステム情報のみでフィルタした視点情報</summary>
        IViewpoint SelfViewSystemInfo { get; }

        /// <summary>自分視点のCO・判定情報を含めてフィルタした視点情報</summary>
        IViewpoint SelfViewTrustInfo { get; }

        /// <summary>自分人外視点のシステム情報のみでフィルタした視点情報</summary>
        IViewpoint SelfRealRoleSystemtInfo { get; }

        /// <summary>自分人外視点のCO・判定情報を含めてフィルタした視点情報　自分が人外でない場合はnull</summary>
        IViewpoint SelfRealRoleViewTrustInfo { get; }

        /// <summary>CO一覧(村陣営COのみ)</summary>
        IEnumerable<ComingOut> ComingOuts { get; }

        /// <summary>占判定一覧</summary>
        IEnumerable<Judge> SeerJudges { get; }

        /// <summary>霊判定一覧</summary>
        IEnumerable<Judge> MediumJudges { get; }


        /// <summary>ゲーム情報の更新</summary>
        void Update(IGameInfo gameInfo);

        /// <summary>視点を再構築する</summary>
        void RebuildViewPoint();
    }
}
