﻿using System;
using System.Collections.Generic;

namespace WhiteWolf.Arrangement
{
    /// <summary>
    /// 特定の視点における内訳パターン
    /// </summary>
    public interface IViewpoint
    {

        /// <summary>視点が取りうる内訳パターン。</summary>
        IEnumerable<IPattern> Patterns { get; }

        /// <summary>
        /// この視点が包含する視点。
        /// この視点から内訳を削除すると、包含する視点からも同じ内訳が削除される。
        /// </summary>
        IReadOnlyCollection<IViewpoint> InclusionViewpoints { get; }

        void AddInclusionViewpoint(IViewpoint viewpoint);

        /// <summary>内訳パターンを削除する。</summary>
        /// <param name="condition">削除する内訳の条件</param>
        void RemovePatterns(Func<IPattern, bool> condition);

        /// <summary>内訳パターンを削除する。</summary>
        /// <param name="condition">削除する内訳の条件</param>
        void RemovePatterns(ICondition condition);

        /// <summary>内訳パターンをリセットする。</summary>
        /// <param name="src">設定する内訳パターン</param>
        void ResetPatterns(IViewpoint src);

        IViewpoint Clone();

    }
}
