﻿using AIWolf.Lib;
using System.Collections.Generic;
using System.Linq;

namespace WhiteWolf.Arrangement
{
    public class IsMatchPattern : ICondition
    {
        public static IEnumerable<Agent> NoCondition => Enumerable.Empty<Agent>();

        public int IsWerewolfBits { get; }

        public int IsPossessedBits { get; }

        public int IsWarewolfSideBits { get; }

        public int IsNotWarewolfSideBits { get; }

        public IsMatchPattern(
            IEnumerable<Agent> warewolves = null,
            IEnumerable<Agent> possessed = null,
            IEnumerable<Agent> warewolfSide = null,
            IEnumerable<Agent> notWarewolfSide = null)
        {
            if (warewolves != null) IsWerewolfBits = warewolves.Sum(a => 1 << (a.AgentIdx - 1));
            if (possessed != null) IsPossessedBits = possessed.Sum(a => 1 << (a.AgentIdx - 1));
            if (warewolfSide != null) IsWarewolfSideBits = warewolfSide.Sum(a => 1 << (a.AgentIdx - 1));
            if (notWarewolfSide != null) IsNotWarewolfSideBits = notWarewolfSide.Sum(a => 1 << (a.AgentIdx - 1));
        }

        public bool IsMatch(IPattern pattern)
        {
            return ((this.IsWerewolfBits & pattern.IsWerewolfBits) == this.IsWerewolfBits)
                && ((this.IsPossessedBits & pattern.IsPossessedBits) == this.IsPossessedBits)
                && ((this.IsWarewolfSideBits & pattern.IsWerewolfTeamBits) == this.IsWarewolfSideBits)
                && ((this.IsNotWarewolfSideBits & pattern.IsNotWerewolfTeamBits) == this.IsNotWarewolfSideBits);
        }

        /// <summary>一致した条件数を取得する。</summary>
        public int GetMatchCount(IPattern pattern)
        {
            var matchCount = 0;
            matchCount += GetBitCount(this.IsWerewolfBits & pattern.IsWerewolfBits);
            matchCount += GetBitCount(this.IsPossessedBits & pattern.IsPossessedBits);
            matchCount += GetBitCount(this.IsWarewolfSideBits & pattern.IsWerewolfTeamBits);
            matchCount += GetBitCount(this.IsNotWarewolfSideBits & pattern.IsNotWerewolfTeamBits);
            return matchCount;
        }

        /// <summary>立っているビットの数を取得する。</summary>
        private int GetBitCount(int value)
        {
            var i = value;
            i = i - ((i >> 1) & 0x55555555);
            i = (i & 0x33333333) + ((i >> 2) & 0x33333333);
            i = (i + (i >> 4)) & 0x0f0f0f0f;
            i = i + (i >> 8);
            i = i + (i >> 16);
            return i & 0x3f;
        }

        #region "Factory系"

        private static Dictionary<Agent, IsMatchPattern> isWolfPatterns = new Dictionary<Agent, IsMatchPattern>();
        /// <summary>
        /// 特定のエージェントが人狼かの条件を取得する。
        /// キャッシュを利用しているため高速。
        /// </summary>
        public static IsMatchPattern GetIsWolfCondition(Agent agent)
        {
            if (!isWolfPatterns.ContainsKey(agent))
            {
                isWolfPatterns[agent] = new IsMatchPattern(warewolves: new List<Agent>() { agent });
            }
            return isWolfPatterns[agent];
        }






        #endregion

    }
}
