﻿using AIWolf.Lib;

namespace WhiteWolf.Arrangement
{
    public class Judge
    {

        /// <summary>判定を出したエージェント</summary>
        public Agent Agent { get; }

        /// <summary>判定を出されたエージェント</summary>
        public Agent Target { get; }

        /// <summary>判定結果</summary>
        public Species Result { get; }

        /// <summary>判定を出した発話</summary>
        public IUtterance JudgeTalk { get; }

        /// <summary>判定を取り消した発話</summary>
        public IUtterance CancelTalk { get; private set; }

        /// <summary>最新データ時点で有効な判定か</summary>
        public bool IsEnable => (CancelTalk == null);


        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="judgeTalk">判定を出した発話</param>
        public Judge(IUtterance judgeTalk)
        {
            Agent = judgeTalk.Agent;
            JudgeTalk = judgeTalk;

            Content content = new Content(JudgeTalk.Text);
            Target = content.Target;
            Result = content.Result;
        }

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="agent">判定を出したエージェント</param>
        /// <param name="target">判定を出されたエージェント</param>
        /// <param name="result">判定結果</param>
        /// <param name="judgeTalk">判定を出した発話</param>
        public Judge(Agent agent, Agent target, Species result, IUtterance judgeTalk)
        {
            Agent = agent;
            Target = target;
            Result = result;
            JudgeTalk = judgeTalk;
        }

        public void Cancel(IUtterance cancelTalk)
        {
            CancelTalk = cancelTalk;
        }

        public override string ToString()
        {
            return Agent.ToString() + " said " + Target.AgentIdx + " is " + Result.ToString();
        }
    }
}
