﻿using AIWolf.Lib;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace WhiteWolf.Arrangement
{
    /// <summary>
    /// 内訳(役職の組み合わせ)パターン
    /// </summary>
    public class Pattern : IPattern
    {

        public IEnumerable<Agent> Wolves { get; }

        public IEnumerable<Agent> Possessed { get; }

        public int IsWerewolfBits { get; }

        public int IsPossessedBits { get; }

        public int IsWerewolfTeamBits { get; }

        public int IsNotWerewolfTeamBits { get; }

        public Pattern(IEnumerable<Agent> wolves, IEnumerable<Agent> possessed)
        {
            this.Wolves = wolves;
            this.Possessed = possessed;

            IsWerewolfBits = wolves.Sum(a => 1 << (a.AgentIdx - 1));
            IsPossessedBits = possessed.Sum(a => 1 << (a.AgentIdx - 1));
            IsWerewolfTeamBits = IsWerewolfBits | IsPossessedBits;
            IsNotWerewolfTeamBits = ~IsWerewolfTeamBits;
        }

        override public string ToString()
        {
            StringBuilder sb = new StringBuilder();

            if (Wolves.Any())
            {
                sb.Append("wolf[" + string.Join(",", Wolves.Select(a => a.AgentIdx)) + "]");
            }

            if (Possessed.Any())
            {
                sb.Append("possessed[" + string.Join(",", Possessed.Select(a => a.AgentIdx)) + "]");
            }

            return sb.ToString();
        }

    }
}
