﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace WhiteWolf.Arrangement
{
    public class Viewpoint : IViewpoint
    {

        private IList<IPattern> patterns;

        public IEnumerable<IPattern> Patterns => patterns;

        private List<IViewpoint> inclusionViewpoints = new List<IViewpoint>();

        IReadOnlyCollection<IViewpoint> IViewpoint.InclusionViewpoints => inclusionViewpoints;

        public void RemovePatterns(Func<IPattern, bool> condition)
        {
            patterns = Patterns.Where(p => !condition(p)).ToList();
            inclusionViewpoints.ForEach(vp => vp.RemovePatterns(condition));
        }

        public void RemovePatterns(ICondition condition)
        {
            patterns = Patterns.Where(p => !condition.IsMatch(p)).ToList();
            inclusionViewpoints.ForEach(vp => vp.RemovePatterns(condition));
        }

        public IViewpoint Clone()
        {
            return new Viewpoint(Patterns);
        }

        public void AddInclusionViewpoint(IViewpoint viewpoint)
        {
            inclusionViewpoints.Add(viewpoint);
        }

        public void ResetPatterns(IViewpoint src)
        {
            patterns = new List<IPattern>(src.Patterns);
            inclusionViewpoints.ForEach(vp => vp.ResetPatterns(src));
        }

        public Viewpoint(IEnumerable<IPattern> patterns)
        {
            this.patterns = new List<IPattern>(patterns);
        }

    }
}
