﻿using AIWolf.Lib;
using System.Collections.Generic;
using System.Linq;

namespace WhiteWolf.Arrangement
{
    public class VoteCollection
    {

        public IReadOnlyDictionary<Agent, Agent> VoteMap { get; }

        public VoteCollection(ICollection<Vote> voteList)
        {
            VoteMap = voteList.ToDictionary(vote => vote.Agent, vote => vote.Target);
        }

        public VoteCollection(IEnumerable<Agent> aliveAgentList, IEnumerable<IContentUtterance> talkList)
        {
            var votemap = aliveAgentList.ToDictionary(agent => agent, agent => (Agent)null);

            // VOTEをANDで繋げたりするのや、2層より深い文法は考慮してない
            foreach (var talk in talkList)
            {
                if (talk.Content.Operator == Operator.NOP && talk.Content.Topic == Topic.VOTE)
                {
                    votemap[talk.Agent] = talk.Content.Target;
                }

                if (talk.Content.Operator == Operator.AND)
                {
                    foreach (var content in talk.Content.ContentList.Where(content => content.Topic == Topic.VOTE))
                    {
                        votemap[talk.Agent] = content.Target;
                    }
                }
            }

            VoteMap = votemap;
        }


    }
}
