﻿using AIWolf.Lib;
using System;
using System.Collections.Generic;
using System.Linq;

namespace WhiteWolf.ContestAgent
{
    public class LearnData
    {
        public Dictionary<Agent, Dictionary<Role, Statistics>> Statistics { get; set; }

        public LearnData(IEnumerable<Agent> agentList)
        {
            var roleList = new List<Role>();
            foreach (Role role in Enum.GetValues(typeof(Role)))
            {
                roleList.Add(role);
            }
            Statistics = agentList.ToDictionary(agent => agent, agent => roleList.ToDictionary(role => role, role => new Statistics()));
        }
    }


    public class Statistics
    {
        public int GameCount { get; set; }
        public int WinCount { get; set; }
        public int LoseCount { get; set; }

        public int SeerCO { get; set; }
        public int MediumCO { get; set; }

        public int FirstDivineIsBlack { get; set; }
        public int FirstDivineIsWhite { get; set; }

        public int FakeCO { get; set; }
        public int Vote { get; set; }
        public int VoteToMax { get; set; }
        public int VoteToNoReceive { get; set; }

        /// <summary>特定のエージェントっぽいか</summary>
        public int[] Player { get; set; } = new int[16];

        public enum PlayerType
        {
            Tomo,
            otsuki,
            takeda,
            simipu,
            Tomato,
            daisyo,
            Camellia,
            HALU,
            cube,
            Sashimi,
            wasabi,
            Udon,
            FoxuFoxu,
            PaSeRi,
            J0hnDoe,
        }

        public int[][][] Talk { get; set; } = new int[16][][];

        public Statistics()
        {
            for (int i = 0; i < 16; i++)
            {
                Talk[i] = new int[21][];
                for (int j = 0; j < 21; j++)
                {
                    Talk[i][j] = new int[6];
                }
            }
        }

        public enum TalkType
        {
            OverAndSkip = 0,
            Judge = 1,
            ComingOut = 2,
            Vote = 3,
            Estimate = 4,
            Other = 5,
        }

    }

}
