﻿using AIWolf.Lib;
using System;
using System.Linq;
using WhiteWolf.ActionPlan;
using WhiteWolf.Arrangement;
using WhiteWolf.Guess;
using WhiteWolf.Talk;

namespace WhiteWolf.ContestAgent
{
    class UdonTalkGenerator : ITalkGenerator
    {
        readonly Random rnd = new Random();

        private LearnData LearnData { get; }

        public UdonTalkGenerator(LearnData learnData)
        {
            LearnData = learnData;
        }


        public string GetTalkText(ITotalityGameInfo gameInfo, ITotalGuess guess, IActionPlan actionPlan)
        {
            // 自分が死んでいたら発話不可能（適当な内容を返す）
            if (!gameInfo.AliveAgentList.Contains(gameInfo.Me))
            {
                return Content.OVER.Text;
            }

            // 占い師固有の動き(他の動きより優先)
            if (gameInfo.MyRole == Role.SEER)
            {
                if (gameInfo.GameSetting.PlayerNum == 5)
                {
                    // 5人村

                    // 初手黒引きのみCO、初手白引きは最後まで潜伏
                    var judge = gameInfo.DayGameInfo[0].DivineResult;
                    if (judge != null && judge.Result == Species.WEREWOLF)
                    {
                        // 占CO
                        if (!gameInfo.ComingOuts.Any(co => co.IsEnable && co.Agent.Equals(gameInfo.Me)))
                        {
                            return new Content(new ComingoutContentBuilder(Agent.NONE, gameInfo.Me, Role.SEER)).Text;
                        }

                        // 占い結果の報告
                        if (!gameInfo.SeerJudges.Any(judge2 => judge2.Agent.Equals(gameInfo.Me) && judge2.JudgeTalk.Day == gameInfo.TodayGameInfo.Day))
                        {
                            if (judge != null)
                            {
                                return new Content(new DivinedResultContentBuilder(Agent.NONE, judge.Target, judge.Result)).Text;
                            }
                        }
                    }
                }
                else
                {
                    // 15人村

                    // 占CO
                    if (!gameInfo.ComingOuts.Any(co => co.IsEnable && co.Agent.Equals(gameInfo.Me)))
                    {
                        return new Content(new ComingoutContentBuilder(Agent.NONE, gameInfo.Me, Role.SEER)).Text;
                    }

                    // 占い結果の報告
                    if (!gameInfo.SeerJudges.Any(judge => judge.Agent.Equals(gameInfo.Me) && judge.JudgeTalk.Day == gameInfo.TodayGameInfo.Day))
                    {
                        var judge = gameInfo.YesterdayGameInfo?.DivineResult;
                        if (judge != null)
                        {
                            return new Content(new DivinedResultContentBuilder(Agent.NONE, judge.Target, judge.Result)).Text;
                        }
                    }
                }
            }

            // 霊能者固有の動き(他の動きより優先)
            if (gameInfo.MyRole == Role.MEDIUM)
            {
                // 霊CO
                if (gameInfo.MyRole == Role.MEDIUM && !gameInfo.ComingOuts.Any(co => co.IsEnable && co.Agent.Equals(gameInfo.Me)))
                {
                    return new Content(new ComingoutContentBuilder(Agent.NONE, gameInfo.Me, Role.MEDIUM)).Text;
                }
                // 霊能結果の報告
                if (gameInfo.YesterdayGameInfo?.ExecuteAgent != null && !gameInfo.MediumJudges.Any(judge => judge.Agent.Equals(gameInfo.Me) && judge.JudgeTalk.Day == gameInfo.TodayGameInfo.Day))
                {
                    var judge = gameInfo.YesterdayGameInfo.MediumResult;
                    return new Content(new IdentContentBuilder(Agent.NONE, judge.Target, judge.Result)).Text;
                }
            }

            // 狂人固有の動き(他の動きより優先) 15人のみ
            if (gameInfo.MyRole == Role.POSSESSED && gameInfo.GameSetting.PlayerNum == 15)
            {
                // 占CO
                if (!gameInfo.ComingOuts.Any(co => co.IsEnable && co.Agent.Equals(gameInfo.Me)))
                {
                    return new Content(new ComingoutContentBuilder(Agent.NONE, gameInfo.Me, Role.SEER)).Text;
                }
                // 占い結果の報告
                if (!gameInfo.SeerJudges.Any(judge => judge.Agent.Equals(gameInfo.Me) && judge.JudgeTalk.Day == gameInfo.TodayGameInfo.Day))
                {
                    var target = actionPlan.DivineTarget;

                    // 5人村初日
                    if (gameInfo.GameSetting.PlayerNum == 5 && gameInfo.TodayGameInfo.Day == 1)
                    {
                        // 対抗に黒出し
                        var otherSeers = gameInfo.ComingOuts.Where(co => co.IsEnable && co.Role == Role.SEER && !co.Agent.Equals(gameInfo.Me)).ToList();
                        if (otherSeers.Count() == 1)
                        {
                            return new Content(new DivinedResultContentBuilder(Agent.NONE, otherSeers.First().Agent, Species.WEREWOLF)).Text;
                        }
                        else
                        {
                            return new Content(new DivinedResultContentBuilder(Agent.NONE, gameInfo.AgentList.Except(otherSeers.Select(co => co.Agent).Append(gameInfo.Me)).First(), Species.WEREWOLF)).Text;
                        }
                    }

                    // 確率で人間か人狼か決める
                    var result = Species.HUMAN;
                    if (rnd.NextDouble() < 0.3)
                    {
                        result = Species.WEREWOLF;
                    }
                    if (gameInfo.GameSetting.PlayerNum == 5 && gameInfo.TodayGameInfo.Day == 1)
                    {
                        result = Species.WEREWOLF;
                    }

                    // 狂人視点で確定白であれば黒を出す
                    if (!gameInfo.SelfRealRoleViewTrustInfo.Patterns.Any(pattern => pattern.Wolves.Contains(target)))
                    {
                        result = Species.WEREWOLF;
                    }

                    // 占い先が人狼確定、もしくは人間確定であれば破綻しない結果にする
                    if (gameInfo.GameSetting.PlayerNum == 15 )
                    {
                        var wolfPatternCount = gameInfo.SelfViewTrustInfo.Patterns.Count(pattern => pattern.Wolves.Contains(target));
                        if (wolfPatternCount == gameInfo.SelfViewTrustInfo.Patterns.Count())
                        {
                            result = Species.WEREWOLF;
                        }
                        if (wolfPatternCount == 0)
                        {
                            result = Species.HUMAN;
                        }
                    }

                    return new Content(new DivinedResultContentBuilder(Agent.NONE, target, result)).Text;
                }
            }

            // 人狼固有の動き(他の動きより優先)
            if (gameInfo.MyRole == Role.WEREWOLF)
            {
                // 5人村PP
                if (gameInfo.GameSetting.PlayerNum == 5 && gameInfo.TodayGameInfo.Day == 2 && !gameInfo.TodayGameInfo.TalkList.Any())
                {
                    var otherSeers = gameInfo.ComingOuts.Where(co => co.IsEnable && co.Role == Role.SEER && !co.Agent.Equals(gameInfo.Me) && gameInfo.AliveAgentList.Contains(co.Agent));
                    if (otherSeers.Count() >= 2)
                    {
                        return new Content(new ComingoutContentBuilder(Agent.NONE, gameInfo.Me, Role.WEREWOLF)).Text;
                    }
                }

                // 5人村占CO
                if (gameInfo.GameSetting.PlayerNum == 5 && LearnData.Statistics[gameInfo.Me][Role.WEREWOLF].GameCount >= 6)
                {
                    if (gameInfo.TodayGameInfo.Day == 1 && gameInfo.TodayGameInfo.TalkList.Count() < 5 && rnd.NextDouble() < 0.8)
                    {
                        return new Content(new ComingoutContentBuilder(Agent.NONE, gameInfo.Me, Role.SEER)).Text;
                    }

                    // 占い結果の報告
                    if (gameInfo.ComingOuts.Any(co => co.Agent.Equals(gameInfo.Me)) &&
                        !gameInfo.SeerJudges.Any(judge => judge.Agent.Equals(gameInfo.Me) && judge.JudgeTalk.Day == gameInfo.TodayGameInfo.Day))
                    {
                        var target = actionPlan.DivineTarget;
                        var result = Species.WEREWOLF;
                        var otherSeers = gameInfo.ComingOuts
                            .Where(co => co.IsEnable && co.Role == Role.SEER && !co.Agent.Equals(gameInfo.Me) && gameInfo.AliveAgentList.Contains(co.Agent))
                            .ToList();
                        if(otherSeers.Count() == 2)
                        {
                            target = otherSeers.First().Agent;
                        }
                        if (gameInfo.TodayGameInfo.Day == 2 && otherSeers.Count() == 1)
                        {
                            target = otherSeers.First().Agent;
                        }
                        return new Content(new DivinedResultContentBuilder(Agent.NONE, target, result)).Text;
                    }
                }
            }

            // 投票先と宣言先が合っていなければ、投票先を宣言する
            var saidVote = new VoteCollection(gameInfo.AliveAgentList, gameInfo.TodayGameInfo.TalkList);
            if (saidVote.VoteMap.ContainsKey(gameInfo.Me))
            {
                var saidTarget = saidVote.VoteMap[gameInfo.Me];
                if (!actionPlan.VoteTarget.Equals(saidTarget))
                {
                    return new Content(new VoteContentBuilder(Agent.NONE, actionPlan.VoteTarget)).Text;
                }
            }

            return Content.OVER.Text;
        }

    }
}
