﻿using AIWolf.Lib;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using WhiteWolf.Arrangement;

namespace WhiteWolf.Guess
{
    /// <summary>
    /// 集計した推理
    /// </summary>
    public interface ITotalGuess
    {

        /// <summary>
        /// 可能性がある内訳パターンと、その尤度
        /// </summary>
        IEnumerable<(IPattern Pattern, double score)> PatternPlausibilities { get; }

        /// <summary>
        /// 最もスコアが高い内訳と、その尤度
        /// </summary>
        IPattern LikelyPattern { get; }

        /// <summary>
        /// エージェント毎の最も人狼スコアが高い内訳と、その尤度
        /// エージェントを含む内訳が存在しない場合、パターンはnull、scoreは0。
        /// </summary>
        ReadOnlyDictionary<Agent, (IPattern Pattern, double score)> LikelyWolfPattern { get; }

        /// <summary>
        /// エージェント毎の最も裏切り者スコアが高い内訳と、その尤度
        /// エージェントを含む内訳が存在しない場合、パターンはnull、scoreは0。
        /// </summary>
        ReadOnlyDictionary<Agent, (IPattern Pattern, double score)> LikelyPossessedPattern { get; }

        /// <summary>
        /// エージェント毎の最も村陣営（人外に含まれない）スコアが高い内訳と、その尤度
        /// エージェントを含む内訳が存在しない場合、パターンはnull、scoreは0。
        /// </summary>
        ReadOnlyDictionary<Agent, (IPattern Pattern, double score)> LikelyVillagerPattern { get; }

    }
}
