﻿using AIWolf.Lib;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using WhiteWolf.Arrangement;

namespace WhiteWolf.Guess
{
    public class TotalGuess : ITotalGuess
    {

        public IEnumerable<(IPattern Pattern, double score)> PatternPlausibilities { get; }

        public IPattern LikelyPattern { get; private set; }

        private Dictionary<Agent, (IPattern Pattern, double score)> likelyWolfPattern
            = new Dictionary<Agent, (IPattern Pattern, double score)>();
        public ReadOnlyDictionary<Agent, (IPattern Pattern, double score)> LikelyWolfPattern { get; private set; }

        public Dictionary<Agent, (IPattern Pattern, double score)> likelyPossessedPattern
            = new Dictionary<Agent, (IPattern Pattern, double score)>();
        public ReadOnlyDictionary<Agent, (IPattern Pattern, double score)> LikelyPossessedPattern { get; private set; }

        public Dictionary<Agent, (IPattern Pattern, double score)> likelyVillagerPattern
            = new Dictionary<Agent, (IPattern Pattern, double score)>();
        public ReadOnlyDictionary<Agent, (IPattern Pattern, double score)> LikelyVillagerPattern { get; private set; }



        public TotalGuess(IEnumerable<Agent> agentList, IEnumerable<(IPattern Pattern, double score)> patternPlausibilities)
        {
            PatternPlausibilities = patternPlausibilities;
            AggregateScore(agentList);
        }


        private void AggregateScore(IEnumerable<Agent> agentList)
        {
            var descPatterns = PatternPlausibilities.OrderByDescending(p => p.score).ToList();

            LikelyPattern = descPatterns.FirstOrDefault().Pattern;

            foreach (Agent agent in agentList)
            {
                // 人狼最大スコア
                likelyWolfPattern[agent] = descPatterns
                    .Where(p => (p.Pattern.IsWerewolfBits & (1 << (agent.AgentIdx - 1))) != 0)
                    .FirstOrDefault();


                // 狂人最大スコア
                likelyPossessedPattern[agent] = descPatterns
                    .Where(p => (p.Pattern.IsPossessedBits & (1 << (agent.AgentIdx - 1))) != 0)
                    .FirstOrDefault();

                // 村側最大スコア
                likelyVillagerPattern[agent] = descPatterns
                    .Where(p => ((p.Pattern.IsWerewolfBits | p.Pattern.IsPossessedBits) & (1 << (agent.AgentIdx - 1))) == 0)
                    .FirstOrDefault();
            }

            LikelyWolfPattern = new ReadOnlyDictionary<Agent, (IPattern Pattern, double score)>(likelyWolfPattern);
            LikelyPossessedPattern = new ReadOnlyDictionary<Agent, (IPattern Pattern, double score)>(likelyPossessedPattern);
            LikelyVillagerPattern = new ReadOnlyDictionary<Agent, (IPattern Pattern, double score)>(likelyVillagerPattern);
        }

    }
}
