﻿using AIWolf.Lib;
using WhiteWolf.ActionPlan;
using WhiteWolf.Arrangement;
using WhiteWolf.Guess;
using WhiteWolf.Talk;

namespace WhiteWolf.Player
{
    public abstract class AbstractWhiteWolfPlayer : IPlayer
    {
        protected virtual ITotalityGameInfo TotallyGameInfo { get; set; }

        protected abstract IGuesser Guesser { get; }

        protected abstract IActionPlanner ActionPlanner { get; }

        protected abstract ITalkGenerator TalkGenerator { get; }

        protected abstract ITalkGenerator WhisperGenerator { get; }

        protected virtual ITotalGuess LatestTotalGuess { get; set; }

        protected virtual IActionPlan LatestActionPlan { get; set; }

        public abstract string Name { get; }

        protected AbstractWhiteWolfPlayer()
        {
        }

        public virtual void Update(IGameInfo gameInfo)
        {
            TotallyGameInfo.Update(gameInfo);
            LatestTotalGuess = Guesser.GetGuess(TotallyGameInfo);
            LatestActionPlan = ActionPlanner.GetPlan(TotallyGameInfo, LatestTotalGuess);
        }

        public virtual void Initialize(IGameInfo gameInfo, IGameSetting gameSetting)
        {
            // TODO Factoryを設定可能にする
            TotallyGameInfo = new TotalityGameInfo(gameInfo, gameSetting);
        }

        public virtual void DayStart()
        {
        }

        public virtual string Talk()
        {
            // memo フレームワーク側で握りつぶすのは良くないのでは
            try
            {
                return TalkGenerator.GetTalkText(TotallyGameInfo, LatestTotalGuess, LatestActionPlan);
            }
            catch (System.Exception)
            {
                return Content.OVER.Text;
            }
        }

        public virtual string Whisper()
        {
            // memo フレームワーク側で握りつぶすのは良くないのでは
            try
            {
                return WhisperGenerator.GetTalkText(TotallyGameInfo, LatestTotalGuess, LatestActionPlan);
            }
            catch (System.Exception)
            {
                return Content.OVER.Text;
            }
        }

        public virtual Agent Vote()
        {
            return LatestActionPlan?.VoteTarget;
        }

        public virtual Agent Attack()
        {
            return LatestActionPlan?.AttackTarget;
        }

        public virtual Agent Divine()
        {
            return LatestActionPlan?.DivineTarget;
        }

        public virtual Agent Guard()
        {
            return LatestActionPlan?.GuardTarget;
        }

        public virtual void Finish()
        {
        }
    }
}
